"""
The colorways.spectra.* modules provide various approximations of 
the visible spectrum. Each provides a wavelen2rgb(nm) function that
takes a wavelength in namometers and returns an RGB color that is 
the approximate color that light at that wavelength appears to us.
"""

